<?php
############################################################
# X-CART-SEO Mod :: http://code.google.com/p/x-cart-seo/
############################################################

# X-Cart Smarty output filter to convert dynamic URL's to bogus static HTML pages
# which will be converted back by mod_rewrite directives

class seo_filter {

	var $_name_delim;
	var $_cur_name;
	var $_keyword_insert;
	var $_printable_insert;
	var $_cat_prefix;
	var $_man_prefix;
	var $_page_prefix;
	var $_prod_prefix;
	var $_pages_prefix;
	var $_norm_match;
	var $_norm_repl;
	var $_max_name_length;

	function seo_filter() {
		global $seo_keyword;

		$this->_name_delim  = '-';
		$this->_keyword_insert = $seo_keyword;
		$this->_printable_insert = $this->_name_delim.'print';
		$this->_cat_prefix  = $this->_name_delim.'c'.$this->_name_delim;
		$this->_man_prefix  = $this->_name_delim.'m'.$this->_name_delim;
		$this->_page_prefix = $this->_name_delim.'p'.$this->_name_delim;
		$this->_prod_prefix = $this->_name_delim.'pr'.$this->_name_delim;
		$this->_pages_prefix = $this->_name_delim.'pg'.$this->_name_delim;
		$this->_norm_match  = array(
									"/[ \/".$this->_name_delim."]+/S",
									"/[^A-Za-z0-9_".$this->_name_delim."]+/S",
									"/[".$this->_name_delim."]+/S"
								  );
		$this->_norm_repl   = array($this->_name_delim, "", $this->_name_delim);
		$this->_max_name_length = 64;
	}
	#
	# Remove/replace characters in names to make valid for use in URL
	#
	function _normalize_name($name) {
		$this->_cur_name = '';
		$name=strtr($name,"",
			"SOZsozYYuAAAAAAACEEEEIIIIDNOOOOOOUUUUYsaaaaaaaceeeeiiiionoooooouuuuyy");
		$this->_cur_name = preg_replace($this->_norm_match, $this->_norm_repl, substr($name, 0, $this->_max_name_length));
		return $this->_cur_name;
	}

	#
	# Generate filename for a category page
	#
	function _category_filename($options, $cat_name) {
		global $sql_tbl;

		if (empty($cat_name))
			$cat_name = func_query_first_cell("SELECT category
											 FROM $sql_tbl[categories]
											 WHERE categoryid='".$options['catid']."'");
		if (empty($cat_name))
			$cat_name = $options['catid'];
		$cat_name  = $this->_normalize_name($cat_name);
		if ($this->_keyword_insert && !substr_count($cat_name,$this->_keyword_insert))
			$cat_name .= $this->_name_delim.$this->_keyword_insert;
		if ($options['print'] && !defined("IS_ROBOT"))
			$cat_name .= $this->_printable_insert;
		if ($options['sort'] && !defined("IS_ROBOT")){
			$cat_name .= $this->_name_delim.$options['sort'];
			if ($options['sortdir']!==false){
				$cat_name .= $this->_name_delim.($options['sortdir']?'down':'up');
			}
		}
		$cat_name .= $this->_cat_prefix.$options['catid'];
		if ($options['page'] <> 1)
			$cat_name .= $this->_page_prefix.$options['page'];

		return $cat_name.'.html';
	}

	#
	# Generate filename for a manufacturers page
	#
	function _manufacturer_filename($options, $man_name=false) {
		global $sql_tbl;

		if (empty($man_name))
			$man_name = func_query_first_cell("SELECT manufacturer
											 FROM $sql_tbl[manufacturers]
											 WHERE manufacturerid='".$options['manid']."'");
		if (empty($man_name))
			$man_name = $options['manid'];
		$man_name = $this->_normalize_name($man_name);
		if ($this->_keyword_insert && !substr_count($man_name,$this->_keyword_insert))
			$man_name .= $this->_name_delim.$this->_keyword_insert;
		if ($options['print'] && !defined("IS_ROBOT"))
			$man_name .= $this->_printable_insert;
		if ($options['sort'] && !defined("IS_ROBOT")){
			$man_name .= $this->_name_delim.$options['sort'];
			if ($options['sortdir']!==false){
				$man_name .= $this->_name_delim.($options['sortdir']?'down':'up');
			}
		}
		$man_name .= $this->_man_prefix.$options['manid'];
		if ($options['page'] <> 1)
		  $man_name .= $this->_page_prefix.$options['page'];

		return $man_name.'.html';
	}

	#
	# Generate filename for a product page
	#
	function _product_filename($options, $prod_name=false) {
		global $sql_tbl;

		if (empty($prod_name))
			$prod_name = func_query_first_cell("SELECT product
											  FROM $sql_tbl[products]
											  WHERE productid = '".$options['prodid']."'");
		if (empty($prod_name))
			$prod_name = $options['prodid'];
		$prod_name = $this->_normalize_name($prod_name);
		if ($this->_keyword_insert && !substr_count($prod_name,$this->_keyword_insert))
			$prod_name .= $this->_name_delim.$this->_keyword_insert;
		if ($options['print'] && !defined("IS_ROBOT"))
			$prod_name .= $this->_printable_insert;
		if ($options['catid'] && !defined("IS_ROBOT")){
			$prod_name .= $this->_cat_prefix.$options['catid'];
			if ($options['page'])
				$prod_name .= $this->_page_prefix.$options['page'];
		}
		$prod_name .= $this->_prod_prefix.$options['prodid'];
		  
		return $prod_name.'.html';
	}

	#
	# Generate filename for a static page
	#
	function _pages_filename($options, $page_name=false) {
		global $sql_tbl;

		if (empty($page_name))
			$page_name = func_query_first_cell("SELECT title
											  FROM $sql_tbl[pages]
											  WHERE pageid = '".$options['pageid']."'");
		if (empty($page_name))
			$page_name = $options['pageid'];
		$page_name = $this->_normalize_name($page_name);
		if ($this->_keyword_insert && !substr_count($page_name,$this->_keyword_insert))
			$page_name .= $this->_name_delim.$this->_keyword_insert;
		if ($options['print'] && !defined("IS_ROBOT"))
			$page_name .= $this->_printable_insert;
		$page_name .= $this->_pages_prefix.$options['pageid'];
		  
		return $page_name.'.html';
	}

	#
	# Category url found callback for preg_replace_callback
	#   $found[1]     <a href="....
	#   not passed    home.php?
	#   $found[2]     cat=###&page=###
	#   $found[3]     "
	#   $found[4]      style="###" title="###">
	#
	function _category_callback($found, $redirect = false) {

		$options = $this->_check_uri($found[2]);
		if($options['catid']){
			if($redirect)
				return $this->_category_filename($options,false);
			else
				return $found[1].$this->_category_filename($options,false). $found[3].$this->_insert_href_title($found[4]);
		}else{
			return $found[0];
		}
	}

	#
	# Manufacturers url found callback for preg_replace_callback
	#   $found[1]     <a href="....
	#   not passed    manufacturers.php?
	#   $found[2]     manufacturerid=###&page=###
	#   $found[3]     "
	#   $found[4]      style="###" title="###">
	#
	function _manufacturer_callback($found, $redirect = false) {

		$options = $this->_check_uri($found[2]);
		if($options['manid']){
			if($redirect)
				return $this->_manufacturer_filename($options,false);
			else
				return $found[1].$this->_manufacturer_filename($options,false). $found[3].$this->_insert_href_title($found[4]);
		}else{
			return $found[0];
		}
	}

	#
	# Product url found callback for preg_replace_callback
	#   $found[1]     <a href="....
	#   not passed    product.php?
	#   $found[2]     productid=###
	#   $found[3]     "
	#   $found[4]      style="###" title="###">
	#
	function _product_callback($found, $redirect = false) {

		$options = $this->_check_uri($found[2]);
		if($options['prodid']){
			if($redirect)
				return $this->_product_filename($options,false);
			else
				return $found[1].$this->_product_filename($options,false). $found[3].$this->_insert_href_title($found[4]);
		}else{
			return $found[0];
		}
	}

	#
	# Page url found callback for preg_replace_callback
	#   $found[1]     <a href="....
	#   not passed    pages.php?
	#   $found[2]     pageid=###
	#   $found[3]     "
	#   $found[4]      style="###" title="###">
	#
	function _pages_callback($found, $redirect = false) {

		$options = $this->_check_uri($found[2]);
		if($options['pageid']){
			if($redirect)
				return $this->_pages_filename($options,false);
			else
				return $found[1].$this->_pages_filename($options,false). $found[3].$this->_insert_href_title($found[4]);
		}else{
			return $found[0];
		}
	}

	#
	# Checks $uri for printable,page,sort,sort_direction
	# Returns array
	#
	function _check_uri($uri){

		$return = array();
		$return['catid'] =  preg_match("/cat=(\d+)/S",$uri, $m)  ? $m[1] : false;
		$return['manid'] =  preg_match("/manufacturerid=(\d+)/S",$uri, $m)  ? $m[1] : false;
		$return['prodid'] = preg_match("/productid=(\d+)/S",$uri, $m) ? $m[1] : false;
		$return['pageid'] = preg_match("/pageid=(\d+)/S",$uri, $m) ? $m[1] : false;
		$return['page'] = preg_match("/page=(\d+)/S",$uri, $m) ? $m[1] : 1;
		$return['sort'] = preg_match("/sort=(\w+)/S",$uri, $m) ? $m[1] : false;
		$return['sortdir'] = preg_match("/sort_direction=([0-1])/S",$uri, $m) ? $m[1] : false;
		$return['print'] = (strpos($uri,"printable=Y")!==false) ? true : false;

		return $return;
	}

	#
	# Adds title="###" text to <a href=""> structures
	# Is passed $found[4]
	#
	function _insert_href_title($found){
		if((strpos(" title=",$found)===false)){
			$found = substr_replace($found,' title="'.str_replace($this->_name_delim,' ',$this->_cur_name).'">',-1);
		}
		return $found;
	}

	#
	# Modify hyperlinks to point to bogus HTML pages.  mod_rewrite will convert back when pages are referenced
	#
	function outputfilter($page_src, &$template_object){

		# Modify links to categories
		$page_src = preg_replace_callback(
					  '/(<a[^<>]+href[ ]*=[ ]*["\']*)[^"\']*home.php\?((?:printable=[^"\'>]+)*cat=[^"\'>]+)(["\'])((?:[^<>]+)*\>)/iUS',
					  array($this,"_category_callback"),
					  $page_src);
		# FancyCategories links
		$page_src = preg_replace_callback(
					  '/(window.location[ ]*=[ ]*["\']*)[^"\']*home.php\?((?:printable=[^"\'>]+)*cat=[^"\'>]+)(["\'])((?:[^<>]+)*\>)/iUS',
					  array($this,"_category_callback"),
					  $page_src);
		# Modify links to manufacturer pages
		$page_src = preg_replace_callback(
					  '/(<a[^<>]+href[ ]*=[ ]*["\']*)[^"\']*manufacturers.php\?((?:printable=[^"\'>]+)*manufacturerid=[^"\'>]+)(["\'])((?:[^<>]+)*\>)/iUS',
					  array($this,"_manufacturer_callback"),
					  $page_src);
		# Modify links to products
		$page_src = preg_replace_callback(
					  '/(<a[^<>]+href[ ]*=[ ]*["\']*)[^"\']*product.php\?((?:printable=[^"\'>]+)*productid=[^"\'>]+)(["\'>])((?:[^<>]+)*\>)/iUS',
					  array($this,"_product_callback"),
					  $page_src);
		# Modify links to pages
		$page_src = preg_replace_callback(
					  '/(<a[^<>]+href[ ]*=[ ]*["\']*)[^"\']*pages.php\?((?:printable=[^"\'>]+)*pageid=[^"\'>]+)(["\'>])((?:[^<>]+)*\>)/iUS',
					  array($this,"_pages_callback"),
					  $page_src);

		return $page_src; 
	}

}
?>